@@ -18,6 +18,17 @@ from email.mime.application import MIMEApplication
 from email.mime.multipart import MIMEMultipart
 from sysconfig import get_path
 import platform
+import ssl
+import shutil
+
+def get_libc_version():
+    import ctypes
+    libc = ctypes.CDLL("libc.so.6")
+    gnu_get_libc_version = libc.gnu_get_libc_version
+    gnu_get_libc_version.restype = ctypes.c_char_p
+    return gnu_get_libc_version().decode()
+
+import logging
 
 try:
     from waflib.Build import CACHE_SUFFIX
@@ -25,6 +36,9 @@ except ImportError:
     sys.path.insert(0, "./third_party/waf")
     from waflib.Build import CACHE_SUFFIX
 
+logging.basicConfig(format='%(asctime)s %(message)s')
+logger = logging.getLogger('autobuild')
+logger.setLevel(logging.INFO)
 
 os.environ["PYTHONUNBUFFERED"] = "1"
 
@@ -104,6 +118,9 @@ if options.retry:
     if options.rebase is None:
         raise Exception('You can only use --retry if you also rebase')
 
+if options.verbose:
+    logger.setLevel(logging.DEBUG)
+
 if options.full_testbase is not None:
     testbase = options.full_testbase
 else:
@@ -130,6 +147,13 @@ def check_symbols(sofile, expected_symbols=""):
     return "objdump --dynamic-syms " + sofile + " | " + \
            "awk \'$0 !~ /" + expected_symbols + "/ {if ($2 == \"g\" && $3 ~ /D(F|O)/ && $4 ~ /(.bss|.text)/ && $7 !~ /(__gcov_|mangle_path)/) exit 1}\'"
 
+def check_versioned_symbol(sofile, symvol, version):
+    return "objdump --dynamic-syms " + sofile + " | " + \
+           "awk \'$7 == \"" + symvol + "\" { " + \
+                "if ($2 == \"g\" && $3 ~ /D(F|O)/ && $4 ~ /(.bss|.text)/ && " + \
+                     "$6 == \"" + version + "\") print $0 }\'" + \
+                "| wc -l | grep -q \'^1$\'"
+
 if args:
     # If we are only running specific test,
     # do not sleep randomly to wait for it to start
@@ -143,7 +167,6 @@ cleanup_list = []
 
 builddirs = {
     "ctdb": "ctdb",
-    "ldb": "lib/ldb",
     "tdb": "lib/tdb",
     "talloc": "lib/talloc",
     "replace": "lib/replace",
@@ -153,16 +176,53 @@ builddirs = {
 }
 
 ctdb_configure_params = " --enable-developer ${PREFIX}"
-samba_configure_params = " ${ENABLE_COVERAGE} ${PREFIX} --with-profiling-data"
+samba_configure_params = " ${ENABLE_COVERAGE} ${PREFIX} --with-profiling-data --with-prometheus-exporter"
+
+# We cannot configure himmelblau on old systems missing openssl 3, with glibc
+# older than version 2.32, or when cargo isn't available.
+himmelblau_configure_params = ''
+
+# To test that waf copes with unknown arguments that look like
+# environment variables, we add a couple of parameters that should be
+# treated environment variables that happen to have no effect.
+#
+# This is for https://bugzilla.samba.org/show_bug.cgi?id=15990: distro
+# build systems do this kind of thing, and older versions of waf
+# allowed it.
+useless_configure_params = " _foobliosity_over_mud=7 GRISHLIHOOD_77=0"
+
+rust_configure_param = ''
+glibc_vers = float('.'.join(get_libc_version().split('.')[:2]))
+cargo = shutil.which('cargo')
+if glibc_vers >= 2.32 and cargo != None:
+    rust_configure_param = ' --enable-rust'
+if ssl.OPENSSL_VERSION_INFO[0] >= 3 and rust_configure_param:
+    himmelblau_configure_params = rust_configure_param + ' --with-himmelblau'
 
 samba_libs_envvars = "PYTHONPATH=${PYTHON_PREFIX}:$PYTHONPATH"
 samba_libs_envvars += " PKG_CONFIG_PATH=$PKG_CONFIG_PATH:${PREFIX_DIR}/lib/pkgconfig"
 samba_libs_envvars += " ADDITIONAL_CFLAGS='-Wmissing-prototypes'"
 samba_libs_configure_base = samba_libs_envvars + " ./configure --abi-check ${ENABLE_COVERAGE} --enable-debug -C ${PREFIX}"
 samba_libs_configure_libs = samba_libs_configure_base + " --bundled-libraries=cmocka,popt,NONE"
-samba_libs_configure_bundled_libs = " --bundled-libraries=!talloc,!pytalloc-util,!tdb,!pytdb,!ldb,!pyldb,!pyldb-util,!tevent,!pytevent,!popt"
+samba_libs_configure_bundled_libs = " --bundled-libraries=!talloc,!pytalloc-util,!tdb,!pytdb,!tevent,!pytevent,!popt"
 samba_libs_configure_samba = samba_libs_configure_base + samba_libs_configure_bundled_libs
 
+is_ubuntu = False
+try:
+    from landscape.lib.os_release import parse_os_release
+    v = parse_os_release()
+    if v["distributor-id"] == "Ubuntu":
+        is_ubuntu = True
+except ImportError:
+    pass
+
+# on ubuntu gcc implies _FORTIFY_SOURCE
+# before 24.04 it was _FORTIFY_SOURCE=2
+# and 24.04 has _FORTIFY_SOURCE=3
+# so we do not specify it explicitly.
+samba_o3_cflags = "-O3"
+if not is_ubuntu:
+    samba_o3_cflags += " -Wp,-D_FORTIFY_SOURCE=2"
 
 def format_option(name, value=None):
     """Format option as str list."""
@@ -238,7 +298,7 @@ tasks = {
     "samba-def-build": {
         "git-clone-required": True,
         "sequence": [
-            ("configure", "./configure.developer" + samba_configure_params),
+            ("configure", "./configure.developer" + samba_configure_params + useless_configure_params),
             ("make", "make -j"),
             ("check-clean-tree", CLEAN_SOURCE_TREE_CMD),
             ("chmod-R-a-w", "chmod -R a-w ."),
@@ -248,7 +308,7 @@ tasks = {
     "samba-mit-build": {
         "git-clone-required": True,
         "sequence": [
-            ("configure", "./configure.developer --with-system-mitkrb5 --with-experimental-mit-ad-dc" + samba_configure_params),
+            ("configure", "./configure.developer --with-system-mitkrb5 --with-experimental-mit-ad-dc --with-systemd-userdb" + samba_configure_params),
             ("make", "make -j"),
             ("check-clean-tree", CLEAN_SOURCE_TREE_CMD),
             ("chmod-R-a-w", "chmod -R a-w ."),
@@ -288,7 +348,7 @@ tasks = {
     "samba-no-opath-build": {
         "git-clone-required": True,
         "sequence": [
-            ("configure", "ADDITIONAL_CFLAGS='-DDISABLE_OPATH=1' ./configure.developer --without-ad-dc " + samba_configure_params),
+            ("configure", "ADDITIONAL_CFLAGS='-DDISABLE_OPATH=1 -DDISABLE_VFS_OPEN_HOW_RESOLVE_NO_SYMLINKS=1 -DDISABLE_PROC_FDS=1' ./configure.developer --without-ad-dc " + samba_configure_params),
             ("make", "make -j"),
             ("check-clean-tree", CLEAN_SOURCE_TREE_CMD),
             ("chmod-R-a-w", "chmod -R a-w ."),
@@ -326,6 +386,7 @@ tasks = {
             "admem_idmap_autorid",
             "ad_member_idmap_ad",
             "ad_member_rfc2307",
+            "ad_member_idmap_nss",
             "ad_member_oneway",
             "chgdcpass",
             "vampire_2000_dc",
@@ -395,6 +456,7 @@ tasks = {
             "admem_idmap_autorid",
             "ad_member_idmap_ad",
             "ad_member_rfc2307",
+            "ad_member_idmap_nss",
             "ad_member_oneway",
             "chgdcpass",
             "vampire_2000_dc",
@@ -506,6 +568,7 @@ tasks = {
             "admem_idmap_autorid",
             "ad_member_idmap_ad",
             "ad_member_rfc2307",
+            "ad_member_idmap_nss",
             "ad_member_offlogon",
             ])),
             ("lcov", LCOV_CMD),
@@ -712,6 +775,7 @@ tasks = {
             "admem_idmap_autorid",
             "ad_member_idmap_ad",
             "ad_member_rfc2307",
+            "ad_member_idmap_nss",
             "ad_member_offlogon",
             ])),
             ("lcov", LCOV_CMD),
@@ -800,7 +864,7 @@ tasks = {
     "samba-o3": {
         "sequence": [
             ("random-sleep", random_sleep(300, 900)),
-            ("configure", "ADDITIONAL_CFLAGS='-O3 -Wp,-D_FORTIFY_SOURCE=2' ./configure.developer --abi-check-disable" + samba_configure_params),
+            ("configure", "ADDITIONAL_CFLAGS='" + samba_o3_cflags + "' ./configure.developer --abi-check-disable" + himmelblau_configure_params + samba_configure_params),
             ("make", "make -j"),
             ("test", make_test(cmd='make test', TESTS="--exclude=selftest/slow-none", include_envs=["none"])),
             ("quicktest", make_test(cmd='make quicktest', include_envs=["ad_dc", "ad_dc_smb1", "ad_dc_smb1_done"])),
@@ -841,7 +905,8 @@ tasks = {
          "./configure.developer ${PREFIX} "
          "--with-selftest-prefix=./bin/ab "
          "--with-cluster-support "
-         "--without-ad-dc "
+         "--with-profiling-data "
+         "--with-prometheus-exporter "
          "--bundled-libraries=!tdb"),
             ("samba-make", "make"),
             ("samba-check", "./bin/smbd --configfile=/dev/null -b | grep CLUSTER_SUPPORT"),
@@ -849,7 +914,7 @@ tasks = {
             ("ctdb-check", "test -e ${PREFIX_DIR}/sbin/ctdbd"),
 
             ("test", make_test(
-                cmd='make test',
+                cmd='PYTHONPATH=${PYTHON_PREFIX}:$PYTHONPATH make test',
                 INJECT_SELFTEST_PREFIX=0,
                 include_envs=["clusteredmember"])
             ),
@@ -867,26 +932,64 @@ tasks = {
             ("talloc-configure", "cd lib/talloc && " + samba_libs_configure_libs),
             ("talloc-make", "cd lib/talloc && make"),
             ("talloc-install", "cd lib/talloc && make install"),
+            ("talloc-abi-check1",
+                check_versioned_symbol(
+                    "./lib/talloc/bin/shared/libtalloc.so.2",
+                    "talloc_named",
+                    "TALLOC_2.0.2"
+                )
+            ),
+            ("talloc-abi-check2",
+                check_versioned_symbol(
+                    "./lib/talloc/bin/shared/libtalloc.so.2",
+                    "talloc_asprintf_addbuf",
+                    "TALLOC_2.3.5"
+                )
+            ),
 
             ("tdb-configure", "cd lib/tdb && " + samba_libs_configure_libs),
             ("tdb-make", "cd lib/tdb && make"),
             ("tdb-install", "cd lib/tdb && make install"),
+            ("tdb-abi-check1",
+                check_versioned_symbol(
+                    "./lib/tdb/bin/shared/libtdb.so.1",
+                    "tdb_errorstr",
+                    "TDB_1.2.1"
+                )
+            ),
+            ("tdb-abi-check2",
+                check_versioned_symbol(
+                    "./lib/tdb/bin/shared/libtdb.so.1",
+                    "tdb_traverse_chain",
+                    "TDB_1.3.17"
+                )
+            ),
 
             ("tevent-configure", "cd lib/tevent && " + samba_libs_configure_libs),
             ("tevent-make", "cd lib/tevent && make"),
             ("tevent-install", "cd lib/tevent && make install"),
+            ("tevent-abi-check1",
+                check_versioned_symbol(
+                    "./lib/tevent/bin/shared/libtevent.so.0",
+                    "_tevent_loop_once",
+                    "TEVENT_0.9.9"
+                )
+            ),
+            ("tevent-abi-check2",
+                check_versioned_symbol(
+                    "./lib/tevent/bin/shared/libtevent.so.0",
+                    "__tevent_req_create",
+                    "TEVENT_0.15.0"
+                )
+            ),
 
-            ("ldb-configure", "cd lib/ldb && " + samba_libs_configure_libs),
-            ("ldb-make", "cd lib/ldb && make"),
-            ("ldb-install", "cd lib/ldb && make install"),
-
-            ("nondevel-configure", samba_libs_envvars + " ./configure ${PREFIX}"),
+            ("nondevel-configure", samba_libs_envvars + " ./configure --private-libraries='!ldb' --vendor-suffix=TEST-STRING~5.1.2 ${PREFIX}"),
             ("nondevel-make", "make -j"),
             ("nondevel-check", "./bin/smbd -b | grep WITH_NTVFS_FILESERVER && exit 1; exit 0"),
+            ("nondevel-check", "./bin/smbd --version | grep -F 'TEST-STRING~5.1.2' && exit 0; exit 1"),
             ("nondevel-no-libtalloc", "find ./bin | grep -v 'libtalloc-report' | grep 'libtalloc' && exit 1; exit 0"),
             ("nondevel-no-libtdb", "find ./bin | grep -v 'libtdb-wrap' | grep 'libtdb' && exit 1; exit 0"),
             ("nondevel-no-libtevent", "find ./bin | grep -v 'libtevent-util' | grep 'libtevent' && exit 1; exit 0"),
-            ("nondevel-no-libldb", "find ./bin | grep -v 'module' | grep -v 'libldbsamba' | grep 'libldb' && exit 1; exit 0"),
             ("nondevel-no-samba-nss_winbind", "ldd ./bin/plugins/libnss_winbind.so.2 | grep 'samba' && exit 1; exit 0"),
             ("nondevel-no-samba-nss_wins", "ldd ./bin/plugins/libnss_wins.so.2 | grep 'samba' && exit 1; exit 0"),
             ("nondevel-no-samba-libwbclient", "ldd ./bin/shared/libwbclient.so.0 | grep 'samba' && exit 1; exit 0"),
@@ -897,19 +1000,24 @@ tasks = {
                 check_symbols("./bin/plugins/libnss_wins.so.2", "_nss_wins_")),
             ("nondevel-no-public-libwbclient",
                 check_symbols("./bin/shared/libwbclient.so.0", "wbc")),
+            ("nondevel-libwbclient-wbcCtxPingDc2@WBCLIENT_0.12",
+                check_versioned_symbol("./bin/shared/libwbclient.so.0", "wbcCtxPingDc2", "WBCLIENT_0.12")),
             ("nondevel-no-public-pam_winbind",
                 check_symbols("./bin/plugins/pam_winbind.so", "pam_sm_")),
             ("nondevel-no-public-winbind_krb5_locator",
                 check_symbols("./bin/plugins/winbind_krb5_locator.so", "service_locator")),
             ("nondevel-no-public-async_dns_krb5_locator",
                 check_symbols("./bin/plugins/async_dns_krb5_locator.so", "service_locator")),
+            ("nondevel-libndr-krb5pac-ndr_pull_PAC_DATA@NDR_KRB5PAC_0.0.1",
+                check_versioned_symbol("./bin/shared/libndr-krb5pac.so.0", "ndr_pull_PAC_DATA", "NDR_KRB5PAC_0.0.1")),
             ("nondevel-install", "make -j install"),
             ("nondevel-dist", "make dist"),
 
             ("prefix-no-private-libtalloc", "find ${PREFIX_DIR} | grep -v 'libtalloc-report' | grep 'private.*libtalloc' && exit 1; exit 0"),
             ("prefix-no-private-libtdb", "find ${PREFIX_DIR} | grep -v 'libtdb-wrap' | grep 'private.*libtdb' && exit 1; exit 0"),
             ("prefix-no-private-libtevent", "find ${PREFIX_DIR} | grep -v 'libtevent-util' | grep 'private.*libtevent' && exit 1; exit 0"),
-            ("prefix-no-private-libldb", "find ${PREFIX_DIR} | grep -v 'module' | grep -v 'libldbsamba' | grep 'private.*libldb' && exit 1; exit 0"),
+            ("prefix-no-private-libldb", "find ${PREFIX_DIR} | grep -v 'module' | grep -v 'libldbsamba' | grep 'private.*libldb.so' && exit 1; exit 0"),
+            ("prefix-public-libldb", "find ${PREFIX_DIR} | grep 'lib/libldb.so' && exit 0; exit 1"),
             ("prefix-no-samba-nss_winbind", "ldd ${PREFIX_DIR}/lib/libnss_winbind.so.2 | grep 'samba' && exit 1; exit 0"),
             ("prefix-no-samba-nss_wins", "ldd ${PREFIX_DIR}/lib/libnss_wins.so.2 | grep 'samba' && exit 1; exit 0"),
             ("prefix-no-samba-libwbclient", "ldd ${PREFIX_DIR}/lib/libwbclient.so.0 | grep 'samba' && exit 1; exit 0"),
@@ -936,7 +1044,6 @@ tasks = {
             ("allshared-no-libtalloc", "find ./bin | grep -v 'libtalloc-report' | grep 'libtalloc' && exit 1; exit 0"),
             ("allshared-no-libtdb", "find ./bin | grep -v 'libtdb-wrap' | grep 'libtdb' && exit 1; exit 0"),
             ("allshared-no-libtevent", "find ./bin | grep -v 'libtevent-util' | grep 'libtevent' && exit 1; exit 0"),
-            ("allshared-no-libldb", "find ./bin | grep -v 'module' | grep -v 'libldbsamba' | grep 'libldb' && exit 1; exit 0"),
             ("allshared-no-samba-nss_winbind", "ldd ./bin/plugins/libnss_winbind.so.2 | grep 'samba' && exit 1; exit 0"),
             ("allshared-no-samba-nss_wins", "ldd ./bin/plugins/libnss_wins.so.2 | grep 'samba' && exit 1; exit 0"),
             ("allshared-no-samba-libwbclient", "ldd ./bin/shared/libwbclient.so.0 | grep 'samba' && exit 1; exit 0"),
@@ -960,7 +1067,7 @@ tasks = {
         "sequence": [
         # build the fuzzers (static) via the oss-fuzz script
             ("fuzzers-mkdir-prefix", "mkdir -p ${PREFIX_DIR}"),
-            ("fuzzers-build", "OUT=${PREFIX_DIR} LIB_FUZZING_ENGINE= SANITIZER=address CXX= CFLAGS= ADDITIONAL_LDFLAGS='-fuse-ld=bfd' ./lib/fuzzing/oss-fuzz/build_samba.sh --enable-afl-fuzzer"),
+            ("fuzzers-build", "OUT=${PREFIX_DIR} LIB_FUZZING_ENGINE= SANITIZER=address CXX= CFLAGS= ADDITIONAL_LDFLAGS='-fuse-ld=bfd' ./lib/fuzzing/oss-fuzz/build_samba.sh --enable-afl-fuzzer --with-profiling-data --with-prometheus-exporter"),
         ],
     },
 
@@ -980,19 +1087,58 @@ tasks = {
 
         # build with all modules static
             ("allstatic-configure", "./configure.developer " + samba_configure_params + " --with-static-modules=ALL"),
-            ("allstatic-make", "make -j"),
+            ("allstatic-make", "nice -n 19 make -j 2"),
             ("allstatic-test", make_test(TESTS="samba3.smb2.create.*nt4_dc")),
             ("allstatic-lcov", LCOV_CMD),
+            ("allstatic-def-check-clean-tree", CLEAN_SOURCE_TREE_CMD),
+            ("allstatic-def-clean", "make clean"),
+
+        # force all libraries as private
+            ("allprivate-def-distclean", "make distclean"),
+            ("allprivate-def-configure", "./configure.developer " + samba_configure_params + " --private-libraries=ALL"),
+            ("allprivate-def-make", "nice -n 19 make -j 2"),
+            # note wrapper libraries need to be public
+            ("allprivate-def-no-public", "ls ./bin/shared | egrep -v '^private$|lib[npqrsu][saueoi][smiscd].*-wrapper.so$|pam_set_items.so|pam_matrix.so' | wc -l | grep -q '^0'"),
+            ("allprivate-def-only-private-ext", "ls ./bin/shared/private | egrep 'private-samba' | wc -l | grep -q '^0' && exit 1; exit 0"),
+            ("allprivate-def-no-non-private-ext", "ls ./bin/shared/private | egrep -v 'private-samba|^libpypamtest.so$' | wc -l | grep -q '^0'"),
+            ("allprivate-def-test", make_test(TESTS="samba3.smb2.create.*nt4_dc")),
+            ("allprivate-def-lcov", LCOV_CMD),
+            ("allprivate-def-check-clean-tree", CLEAN_SOURCE_TREE_CMD),
+            ("allprivate-def-clean", "make clean"),
+
+        # force all libraries as private with a non default
+        # extension and 2 exceptions
+            ("allprivate-ext-distclean", "make distclean"),
+            ("allprivate-ext-configure", "./configure.developer " + samba_configure_params + " --private-libraries=ALL --private-library-extension=private-library --private-extension-exception=pac,ndr"),
+            ("allprivate-ext-make", "nice -n 19 make -j 2"),
+            # note wrapper libraries need to be public
+            ("allprivate-ext-no-public", "ls ./bin/shared | egrep -v '^private$|lib[npqrsu][saueoi][smiscd].*-wrapper.so$|pam_set_items.so|pam_matrix.so' | wc -l | grep -q '^0'"),
+            ("allprivate-ext-no-private-default-ext", "ls ./bin/shared/private | grep 'private-samba' | wc -l | grep -q '^0'"),
+            ("allprivate-ext-has-private-ext", "ls ./bin/shared/private | grep 'private-library' | wc -l | grep -q '^0' && exit 1; exit 0"),
+            ("allprivate-ext-libndr-no-private-ext", "ls ./bin/shared/private | grep -v 'private-library' | grep 'libndr' | wc -l | grep -q '^1'"),
+            ("allprivate-ext-libpac-no-private-ext", "ls ./bin/shared/private | grep -v 'private-library' | grep 'libpac' | wc -l | grep -q '^1'"),
+            ("allprivate-ext-test", make_test(TESTS="samba3.smb2.create.*nt4_dc")),
+            ("allprivate-ext-lcov", LCOV_CMD),
+            ("allprivate-ext-check-clean-tree", CLEAN_SOURCE_TREE_CMD),
+            ("allprivate-ext-clean", "make clean"),
 
         # retry with nonshared smbd and smbtorture
             ("nonshared-distclean", "make distclean"),
             ("nonshared-configure", "./configure.developer " + samba_configure_params + " --bundled-libraries=ALL --with-static-modules=ALL --nonshared-binary=smbtorture,smbd/smbd"),
-            ("nonshared-make", "make -j"),
-            # TODO ("nonshared-test", make_test(TESTS="samba3.smb2.create.*nt4_dc")),
-            # TODO ("nonshared-lcov", LCOV_CMD),
+            ("nonshared-make", "nice -n 19 make -j 2"),
+            ("nonshared-test", make_test(TESTS="samba3.smb2.create.*nt4_dc")),
+            ("nonshared-lcov", LCOV_CMD),
+            ("nonshared-check-clean-tree", CLEAN_SOURCE_TREE_CMD),
+            ("nonshared-clean", "make clean"),
 
-            ("check-clean-tree", CLEAN_SOURCE_TREE_CMD),
-            ("clean", "make clean"),
+        # retry without winbindd
+            ("nonwinbind-distclean", "make distclean"),
+            ("nonwinbind-configure", "./configure.developer " + samba_configure_params + " --bundled-libraries=ALL --with-static-modules=ALL --without-winbind"),
+            ("nonwinbind-make", "nice -n 19 make -j 2"),
+            ("nonwinbind-test", make_test(TESTS="samba3.smb2.*.simpleserver")),
+            ("nonwinbind-lcov", LCOV_CMD),
+            ("nonwinbind-check-clean-tree", CLEAN_SOURCE_TREE_CMD),
+            ("nonwinbind-clean", "make clean"),
         ],
     },
 
@@ -1000,7 +1146,7 @@ tasks = {
         "sequence": [
             ("random-sleep", random_sleep(300, 900)),
 
-            ("configure", "./configure.developer ${ENABLE_COVERAGE} ${PREFIX} --with-profiling-data --disable-python --without-ad-dc"),
+            ("configure", "./configure.developer " + samba_configure_params + " --disable-python --without-ad-dc"),
             ("make", "make -j"),
             ("find-python", "script/find_python.sh ${PREFIX}"),
             ("test", "make test-nopython"),
@@ -1020,10 +1166,6 @@ tasks = {
             ("tevent-make", "cd lib/tevent && make"),
             ("tevent-install", "cd lib/tevent && make install"),
 
-            ("ldb-configure", "cd lib/ldb && " + samba_libs_configure_base + " --bundled-libraries=cmocka,NONE --disable-python"),
-            ("ldb-make", "cd lib/ldb && make"),
-            ("ldb-install", "cd lib/ldb && make install"),
-
         # retry against installed library packages, but no required modules
             ("libs-configure", samba_libs_configure_base + samba_libs_configure_bundled_libs + " --disable-python --without-ad-dc  --with-static-modules=!FORCED,!DEFAULT --with-shared-modules=!FORCED,!DEFAULT"),
             ("libs-make", "make -j"),
@@ -1034,29 +1176,10 @@ tasks = {
         ],
     },
 
-    "samba-shellcheck": {
+    "samba-codecheck": {
         "sequence": [
             ("run", "script/check-shell-scripts.sh ."),
-        ],
-    },
-
-    "ldb": {
-        "sequence": [
-            ("random-sleep", random_sleep(60, 600)),
-            ("configure", "./configure ${ENABLE_COVERAGE} --enable-developer -C ${PREFIX}"),
-            ("make", "make"),
-            ("install", "make install"),
-            ("test", "make test"),
-            ("lcov", LCOV_CMD),
-            ("clean", "make clean"),
-            ("configure-no-lmdb", "./configure ${ENABLE_COVERAGE} --enable-developer --without-ldb-lmdb -C ${PREFIX}"),
-            ("make-no-lmdb", "make"),
-            ("test-no-lmdb", "make test"),
-            ("lcov-no-lmdb", LCOV_CMD),
-            ("install-no-lmdb", "make install"),
-            ("check-clean-tree", CLEAN_SOURCE_TREE_CMD),
-            ("distcheck", "make distcheck"),
-            ("clean", "make clean"),
+            ("run", "script/codespell.sh ."),
         ],
     },
 
@@ -1186,16 +1309,22 @@ if os.environ.get("AUTOBUILD_SKIP_SAMBA_O3", "0") == "1":
 
 
 def do_print(msg):
-    print("%s" % msg)
+    logger.info(msg)
+    sys.stdout.flush()
+    sys.stderr.flush()
+
+def do_debug(msg):
+    logger.debug(msg)
     sys.stdout.flush()
     sys.stderr.flush()
 
 
 def run_cmd(cmd, dir=".", show=None, output=False, checkfail=True):
     if show is None:
-        show = options.verbose
-    if show:
+        do_debug("Running: '%s' in '%s'" % (cmd, dir))
+    elif show:
         do_print("Running: '%s' in '%s'" % (cmd, dir))
+
     if output:
         out = check_output([cmd], shell=True, cwd=dir)
         return out.decode(encoding='utf-8', errors='backslashreplace')
@@ -1233,9 +1362,8 @@ class builder(object):
         self.next = 0
         self.stdout_path = "%s/%s.stdout" % (gitroot, self.tag)
         self.stderr_path = "%s/%s.stderr" % (gitroot, self.tag)
-        if options.verbose:
-            do_print("stdout for %s in %s" % (self.name, self.stdout_path))
-            do_print("stderr for %s in %s" % (self.name, self.stderr_path))
+        do_debug("stdout for %s in %s" % (self.name, self.stdout_path))
+        do_debug("stderr for %s in %s" % (self.name, self.stderr_path))
         run_cmd("rm -f %s %s" % (self.stdout_path, self.stderr_path))
         self.stdout = open(self.stdout_path, 'w')
         self.stderr = open(self.stderr_path, 'w')
@@ -1252,12 +1380,12 @@ class builder(object):
             assert "dependency" not in definition
 
     def mark_existing(self):
-        do_print('%s: Mark as existing dependency' % self.name)
+        do_debug('%s: Mark as existing dependency' % self.name)
         self.next = len(self.sequence)
         self.done = True
 
     def add_consumer(self, consumer):
-        do_print("%s: add consumer: %s" % (self.name, consumer.name))
+        do_debug("%s: add consumer: %s" % (self.name, consumer.name))
         consumer.producer = self
         consumer.test_source_dir = self.test_source_dir
         self.consumers.append(consumer)
@@ -1265,7 +1393,7 @@ class builder(object):
     def start_next(self):
         if self.producer is not None:
             if not self.producer.done:
-                do_print("%s: Waiting for producer: %s" % (self.name, self.producer.name))
+                do_debug("%s: Waiting for producer: %s" % (self.name, self.producer.name))
                 return
 
         if self.next == 0:
@@ -1355,9 +1483,9 @@ class buildlist(object):
 
         tasknames = implicit_tasknames.copy()
         tasknames.extend(given_tasknames)
-        do_print("given_tasknames: %s" % given_tasknames)
-        do_print("implicit_tasknames: %s" % implicit_tasknames)
-        do_print("tasknames: %s" % tasknames)
+        do_debug("given_tasknames: %s" % given_tasknames)
+        do_debug("implicit_tasknames: %s" % implicit_tasknames)
+        do_debug("tasknames: %s" % tasknames)
         self.tlist = [builder(n, tasks[n]) for n in tasknames]
 
         if options.retry:
@@ -1393,11 +1521,11 @@ class buildlist(object):
                     b.mark_existing()
 
         for b in self.tlist:
-            do_print("b.name=%s" % b.name)
+            do_debug("b.name=%s" % b.name)
             if "dependency" not in b.definition:
                 continue
             depname = b.definition["dependency"]
-            do_print("b.name=%s: dependency:%s" % (b.name, depname))
+            do_debug("b.name=%s: dependency:%s" % (b.name, depname))
             for p in self.tlist:
                 if p.name == depname:
                     p.add_consumer(b)
@@ -1673,14 +1801,16 @@ The top commit for the tree that was built was:
 
 ''' % (log_base, failed_tag, log_base, failed_tag, log_base, top_commit_msg)
 
-    if add_log_tail:
-        f = open("%s/%s.stdout" % (gitroot, failed_tag), 'r')
+    log_stdout = "%s/%s.stdout" % (gitroot, failed_tag)
+    if add_log_tail and os.access(log_stdout, os.R_OK):
+        f = open(log_stdout, 'r')
         lines = f.readlines()
         log_tail = "".join(lines[-50:])
         num_lines = len(lines)
-        if num_lines < 50:
+        log_stderr = "%s/%s.stderr" % (gitroot, failed_tag)
+        if num_lines < 50 and os.access(log_stderr, os.R_OK):
             # Also include stderr (compile failures) if < 50 lines of stdout
-            f = open("%s/%s.stderr" % (gitroot, failed_tag), 'r')
+            f = open(log_stderr, 'r')
             log_tail += "".join(f.readlines()[-(50 - num_lines):])
 
         text += '''
